from threading import Thread

from resources.lib.const import SERVICE, SETTINGS, DOWNLOAD_SERVICE_EVENT
from resources.lib.kodilogging import logger
from resources.lib.services import Service
from resources.lib.storage.settings import settings
from resources.lib.storage.sqlite import SQLiteStorage, DB
from resources.lib.utils.download import download


class DownloadService(Service):
    SERVICE_NAME = SERVICE.DOWNLOAD_SERVICE

    def __init__(self, api, threads):
        super(DownloadService, self).__init__()
        self._api = api
        self._event_callbacks = {
            DOWNLOAD_SERVICE_EVENT.ITEM_ADDED: self.process_queue,
            DOWNLOAD_SERVICE_EVENT.ITEM_RESUMED: self.resume_download_item
        }
        self.db = SQLiteStorage.Download()
        self.threads = threads

    def process_queue(self, download_id):
        logger.debug("Starting download of %s" % download_id)
        records = self.db.get_queued()
        self.download_many(records)

    def download_many(self, records):
        for row in records:
            self.download(row)

    def download(self, item):
        dl_id = item[0]
        download_folder = item[1]
        url = item[2]
        name = item[3]
        worker = Thread(target=download,
                        args=(url, download_folder, name, dl_id))
        self.threads.append(worker)
        worker.start()

    def is_all_done(self):
        records = self.db.get_downloading()
        return len(records) > 0

    def resume_download(self):
        records = self.db.get_downloading()
        self.download_many(records)

    def resume_download_item(self, download_id):
        item = DB.DOWNLOAD.get(download_id)
        self.download(item)
